#include <SDL.h>
#include <SDL_thread.h>
#include <SDL_image.h>

#include <stdexcept>

#include "ImageFactory.h"
#include "ThumbProvider.h"

ImageFactory * ImageFactory::factory = 0;

ImageFactory::ImageFactory() : requests()
{
}

void ImageFactory::doLoad(ImageIdentifier path,ImageUser & user)
{
    ImageProvider * imp;
    InternalTask::lock();
    
    if (!(requests.find(path) != requests.end())) {
	ImageProvider * nv;
	if (!path.isThumb) {
	    nv = new ImageProvider(path.name);
	    nv->autoSignal();
	} else {
	    ThumbProvider * t;
	    t = new ThumbProvider(path.name,path.sx,path.sy);
	    t->init();
	    nv = t;
	}
	requests[path] = nv;
    }
    imp = requests[path];
    user.bind(imp);

    InternalTask::unlock();
}

void ImageFactory::cleanUp()
{
    static bool running = false;
    bool deleted = false;
    InternalTask::lock();
    if (running) {
	InternalTask::unlock();
	return;
    }
    running = true;
    do {
	deleted = false;
	for(typeof(requests.begin()) it = requests.begin(); it != requests.end(); ++it) {
	    
	    if ((!it->second->isRunning()) && (!it->second->used())) {
		jsrlog("removing unused image(%#x) %s(%d)\n",(it->second),it->first.name.c_str(),it->first.isThumb);
		delete it->second;
		requests.erase(it->first);
		deleted = true;
		break;
	    }
	}
    } while(deleted);
    running = false;
    InternalTask::unlock();
}

void ImageFactory::loadImage(std::string path, ImageUser & user)
{
    if (!factory) {
	factory = new ImageFactory();
    }
    factory->doLoad(ImageIdentifier(path),user);
}

void ImageFactory::loadThumb(std::string path, int sx, int sy, ImageUser & user)
{
    if (!factory) {
	factory = new ImageFactory();
    }
    factory->doLoad(ImageIdentifier(path,sx,sy),user);
}

void ImageFactory::invalidateImage(const std::string & n)
{
    InternalTask::lock();
    for(typeof(factory->requests.begin()) it = factory->requests.begin(); it != factory->requests.end(); ++it) {
	if (it->first.name == n) {
	    it->second->reload();
	}
    }
    InternalTask::unlock();
}
