#include "ImageLoader.h"

ImageLoader::ImageLoader() : JSNotifiable(), path()
{
    imageUser = new ImageUser();
    imageUser->setTarget(this);
    loading = false;
    loadResult = false;
    sizeX = -1;
    sizeY = -1;
    forceChangeTrigger = false;
}

ImageLoader::~ImageLoader()
{
    delete(imageUser);
}

bool ImageLoader::updateStatus()
{
    ImageProvider * ip = imageUser->getProducer();
    bool change;
    bool nvLoading;
    bool nvLoadResult;
    int nvSizeX,nvSizeY;
    
    if (ip == 0) {
	jsrlog("no producer\n");
	nvLoading = false;
	nvLoadResult = false;
	nvSizeX = -1;
	nvSizeY = -1;
    } else {
	if (!ip->isDone()) {
	    nvLoading = true;
	    nvLoadResult = false;
	    nvSizeX = -1;
	    nvSizeY = -1;
	} else {
	    nvLoading = false;
	    
	    Surface * s = imageUser->getSurface();
	    if ((s)&&(s->GetWidth() > 0)) {
		jsrlog("Image successfully loaded\n");
		nvLoadResult = true;
		nvSizeX = s->GetWidth();
		nvSizeY = s->GetHeight();
	    } else {
		nvLoadResult = false;
		nvSizeX = -1;
		nvSizeY = -1;
	    }
	}
    }
    jsrlog("Status id %d %d %d %d\n",nvLoading,nvLoadResult,nvSizeX,nvSizeY);
    change = forceChangeTrigger || (loading != nvLoading) || (loadResult != nvLoadResult)
	    || (sizeX != nvSizeX) || (sizeY != nvSizeY);
    forceChangeTrigger = false;
    jsrlog("change = %d\n",change);
    loading = nvLoading;
    loadResult = nvLoadResult;
    sizeX = nvSizeX;
    sizeY = nvSizeY;
    return change;
}

void ImageLoader::load(const std::string & nvpath)
{
    path = nvpath;
    ImageFactory::loadImage(path,*imageUser);
    loading = true;
    loadResult = false;
    sizeX = -1;
    sizeY = -1;
    forceChangeTrigger = true;
}

bool ImageLoader::getLoading() const
{
    return loading;
}

bool ImageLoader::getLoadResult() const
{
    return loadResult;
}

int ImageLoader::getSizeX() const
{
    return sizeX;
}

int ImageLoader::getSizeY() const
{
    return sizeY;
}

std::string ImageLoader::getPath() const
{
    return path;
}

ImageLoader * newImageLoader()
{
    return new ImageLoader();
}

JS_EXPORT_CLASS(ImageLoader,JSNotifiable);

JS_RO_PROPERTY(ImageLoader,sizeX,getSizeX,int,"%i",0);
JS_RO_PROPERTY(ImageLoader,sizeY,getSizeY,int,"%i",0);
JS_RO_PROPERTY(ImageLoader,loading,getLoading,bool,"%b",0);
JS_RO_PROPERTY(ImageLoader,loadResult,getLoadResult,bool,"%b",0);
JS_RO_PROPERTY(ImageLoader,path,getPath,JSReflection::JS_String,"%s",0);
JS_NOCONTEXT_OBJECT_VOID_FUNCTION_1(ImageLoader,load,JSReflection::JS_String,"%s",0);
JS_NOCONTEXT_GLOBAL_FUNCTION_0(newImageLoader,ImageLoader*,"%x",ImageLoader::ClassInfo)
