#include <SDL.h>
#include <SDL_thread.h>
#include <SDL_image.h>

#include <stdexcept>

#include "ImageFactory.h"

ImageUser::ImageUser(TaskQueue * tq) : Notifier(tq)
{
    current = 0;
    producer = 0;
    serial = 0;
}

ImageUser::~ImageUser()
{
    bind(0);
}

Surface * ImageUser::getSurface()
{
    if (producer) {
	bool changed;
	current = producer->updateSurface(current, changed);
	if (changed) {
	    serial++;
	}
    }
    return current;
}

Surface * ImageUser::getFinalSurface()
{
    if (producer) {
	bool changed;
	current = producer->updateSurface(current, changed);
	if (changed) {
	    serial++;
	}
    }
    return current;
}

int ImageUser::getSerial() const
{
    return serial;
}

void ImageUser::bind(ImageProvider * nvproducer)
{
    if (producer == nvproducer) {
	return;
    }
    InternalTask::lock();
    serial++;

    if (producer) {
	producer->releaseSurface(current);
	producer->removeUser(this);
    }
    
    if (nvproducer) {
	nvproducer->addUser(this);
    }
    current = 0;
    producer = nvproducer;
    getSurface();
    ImageFactory::factory->cleanUp();
    InternalTask::unlock();
}

bool ImageUser::binded() const
{
    return producer != 0;
}

ImageProvider * ImageUser::getProducer() const
{
    return producer;
}

