#ifndef IMAGEWIDGET_H
#define IMAGEWIDGET_H 1

#include "Widget.h"

class ImageWidgetCoordinates {
    friend class ImageWidget;
    bool exists;
    /// pt coords with 8 bit for decimal.
    int ptcoords[4][2];
    int destw,desth;
    int surfw,surfh;
    
public:
    Surface * surface;
    int surfSerial;
    ImageWidgetCoordinates();
    
    bool compare(const ImageWidgetCoordinates & ref) const{
	if (!ref.exists) {
	    return false;
	}
	if (!exists) {
	    return false;
	}
	for(int i=0;i<4;++i) {
	    for(int j=0;j<2;++j) {
		if (ptcoords[i][j] != ref.ptcoords[i][j]) {
		    return false;
		}
	    }
	}
	if (surfw != ref.surfw) {
	    return false;
	}
	if (surfh != ref.surfh) {
	    return false;
	}
	 
	if (destw != ref.destw) {
	    return false;
	}
	if (desth != ref.desth) {
	    return false;
	}
	if (surfSerial != ref.surfSerial) {
	    return false;
	}
	return true;
    }
};

class ImageWidget : public Widget
{
    JS_EXPORTED_CLASS
    
    /// Center : coord of the widget center in the original image
    /// 0 -> image center. -65536 => image border 1, 65536 => image opposite border
    int centerX,centerY;
    /// Angle  : 0 & 360 mean no rotation 
    int angle;
    /// Zoom : 256 mean best feet. 512 mean bestfeet x2, ...
    int zoom;
    
    ImageWidgetCoordinates lastdraw;
    
    ImageWidgetCoordinates prepareDraw();
    
    double getCos() const;
    double getSin() const;
    
    void createSurface(SDL_Surface * org,SDL_Surface * dest);
    Surface * currentSurface;
    
    ImageUser image;
    std::string image_file;
    
    virtual void globalVisibleChanged();
    
    virtual void notified(Notifier * v);
    
public:
    ImageWidget();
    virtual ~ImageWidget();
    
    int getCenterX() const;
    void setCenterX(int cx);
    
    int getCenterY() const;
    void setCenterY(int cy);
    
    int getAngle() const;
    void setAngle(int a);
    
    int getZoom() const;
    void setZoom(int z);
    
    // horizontal vector.
    int getVectorXX() ;
    int getVectorXY() ;
    // vertical vector.
    int getVectorYX() ;
    int getVectorYY() ;
    
    void setImage(const std::string& s);
    std::string getImage() const;
    
    virtual void drawSelf(Painter & p);
};

#endif

