#ifndef LAYOUT_H
#define LAYOUT_H 1

#include "jsreflect.h"
#include "Widget.h"

#include <string>
#include <vector>
#include <list>

class GridLayout;

class LayoutCell : public JSReflection::Object
{
    JS_EXPORTED_CLASS
    
    friend class GridLayout;
    friend class LayoutLine;
    
public:
    JSReflection::JSSourceInfo definition;
protected:
    int width;
    int left_border;
    int right_border;
    
    std::string str_width;
    std::string str_left_border;
    std::string str_right_border;
    
    GridLayout * layout;
    
    void update();
    
public:
    LayoutCell();
    virtual ~LayoutCell();
    
    int getWidth() const;
    void setWidth(int i);
    void setStrWidth(const std::string & jsval);
    
    int getLeftBorder() const;
    void setLeftBorder(int i);
    void setStrLeftBorder(const std::string & jsval);
    
    int getRightBorder() const;
    void setRightBorder(int i);
    void setStrRightBorder(const std::string & jsval);
    
    Widget * getArea() const;
};

class LayoutLine : public JSReflection::Object
{
    JS_EXPORTED_CLASS
    
    friend class GridLayout;
public:
    JSReflection::JSSourceInfo definition;
protected:
    
    int height;
    int top_border;
    int bottom_border;
    
    std::string str_height;
    std::string str_top_border;
    std::string str_bottom_border;
    
    GridLayout * layout;
    std::vector<LayoutCell*> cells;
    
    void update();
    
public:
    LayoutLine();
    virtual ~LayoutLine();
    
    int getHeight() const;
    void setHeight(int i);
    void setStrHeight(const std::string & jsval);
    
    int getTopBorder() const;
    void setTopBorder(int i);
    void setStrTopBorder(const std::string & jsval);
    
    int getBottomBorder() const;
    void setBottomBorder(int i);
    void setStrBottomBorder(const std::string & jsval);
    
    void addCell(LayoutCell * cell);
    
    int getCellCount() const;
    LayoutCell * getCell(int i) const;
    
    Widget * getArea() const;
};

class Layout: public JSReflection::Object
{
    JS_EXPORTED_CLASS
    friend class Widget;
protected:
    Widget * widget;
public:
    Layout();
    virtual ~Layout();
    
    virtual void addWidget(Widget * v) = 0;
    
    virtual void relayoutWidget(Widget * v) = 0;
    
    virtual void update() = 0;
    
    Widget * getArea() const;
};

class FullLayout : public Layout
{
    JS_EXPORTED_CLASS
    int hmargin,vmargin;
public:
    FullLayout();
    virtual ~FullLayout();
    
    virtual void addWidget(Widget * v);
    
    virtual void relayoutWidget(Widget * v);
    
    virtual void update();
    
    int getHMargin() const { return hmargin; }
    int getVMargin() const { return vmargin; }
    void setHMargin(int i) { hmargin = i; }
    void setVMargin(int i) { vmargin = i; }
    
};

class GridLayout: public Layout
{
protected:
    std::vector<LayoutLine*> lines;
    int curline,curcell;
    
public:
    GridLayout();
    virtual ~GridLayout();
    
    void addLine(LayoutLine * l);
    
    virtual void addWidget(Widget * v);
    
    virtual void relayoutWidget(Widget * v);
    
    virtual void update();
};

#endif
