#ifndef RENDER_H
#define RENDER_H 1

#include <list>
class RendererItem;

class Renderer {
    friend class RendererItem;
    void findCollision(std::list<RendererItem*> & intersect,int x,int y,int w,int h) const;
    int nextFreePlaceOnRight(int startx,int starty,int w,int h) const;
    
    void terminateLine();
    void addItemToLine(RendererItem * gi);
    
public:
    
    void layout();
    
    int areaWidth;
    
    /// List of all items
    std::list<RendererItem*> items;
    
    /// List of floating items
    std::list<RendererItem*> floating;
    
    /// Items in the current line
    std::list<RendererItem*> currentLine;
    
    /// y (top) pos of the current line
    int currentLineTop;
    /// x (from left) pos of the current text
    int currentLineX;
    /// height of the current line
    int currentLineAscent,currentLineDescent;
    /// Size of currentLineSkip
    int currentLineSkip;
    
    int currentLineAlignement;
};

class RendererItem {
    friend class Renderer;
protected:
    Renderer * renderer;
public:
    int x,y,w,h;
    int ascent;
    int descent;
    int lineskip;
    
    RendererItem(Renderer * r);
    virtual ~RendererItem();
    virtual void updateSize();
    
    /// get the preferred horizontal location (for floating)
    virtual int getPreferredLocation() const;
    
    /// For spaces, should the item be removed if placed on a line start / end?
    virtual bool allowedOnLineBound() const;
    
    
};

class InlineItem {
protected:
    
    virtual bool floating() const;
};

#endif

