#include <SDL.h>
#include <SDL_image.h>
#include "Graphs.h"

#include <assert.h>

#include <stdio.h>
#include <stdlib.h>

static unsigned char tbl[4 * 65536];
static bool tableok = false;

static void calcTable();

static void getFactors(int a1,int a2, int & ra,int & m1,int & m2)
{
    int idx = (a1 << 10) | (a2 <<2);
    
    ra = tbl[idx];
    m1 = tbl[idx + 1];
    m2 = tbl[idx + 2];
}

static void getCompose(unsigned char r1,unsigned char g1,unsigned char b1,unsigned char a1,
                        unsigned char r2,unsigned char g2,unsigned char b2,unsigned char a2,
                        unsigned char & rr,unsigned char & rg,unsigned char & rb,unsigned char & ra )
{
    int m1,m2,alpha;
    
    getFactors(a1,a2,alpha,m1,m2);
    
    ra = alpha;
    rr = (r1 * m1 + r2 * m2) >> 7;
    rg = (g1 * m1 + g2 * m2) >> 7;
    rb = (b1 * m1 + b2 * m2) >> 7;
}


static void Compose_RGBA_RGB(SDL_Surface *src, SDL_Rect *srcrect, SDL_Surface *dst, SDL_Rect *dstrect,unsigned char srcalpha)
{
    unsigned char * src_ptr, * src_line_ptr;
    unsigned char * dst_ptr, * dst_line_ptr;
    int width, height;
    int srcbpp = src->format->BytesPerPixel;

    assert(dst->format->BytesPerPixel == 4);
    
    SDL_LockSurface(src);
    SDL_LockSurface(dst);
    
    dst_line_ptr = (unsigned char*)dst->pixels + dstrect->y * dst->pitch + dstrect->x * 4;
    src_line_ptr = (unsigned char*)src->pixels + srcrect->y * src->pitch + srcrect->x * 4;
    height = srcrect->h;
    
    
    while ( height-- ) {
	src_ptr = src_line_ptr;
	dst_ptr = dst_line_ptr;
	
	width = srcrect->w;
	
	for ( int c=width; c; --c ) {
	    getCompose(dst_ptr[0],dst_ptr[1],dst_ptr[2],dst_ptr[3],
			src_ptr[0],src_ptr[1],src_ptr[2],srcalpha,
	    		dst_ptr[0],dst_ptr[1],dst_ptr[2],dst_ptr[3]);
	    //dst_ptr[3] = 80;
	    src_ptr += srcbpp;
	    dst_ptr += 4;
	}
	src_line_ptr += src->pitch;
	dst_line_ptr += dst->pitch;
    }
    SDL_UnlockSurface(src);
    SDL_UnlockSurface(dst);
}

static void Compose_RGBA_RGBA(SDL_Surface *src, SDL_Rect *srcrect, SDL_Surface *dst, SDL_Rect *dstrect)
{
    unsigned char * src_ptr, * src_line_ptr;
    unsigned char * dst_ptr, * dst_line_ptr;
    int width, height;

    assert(src->format->BytesPerPixel == 4);
    assert(dst->format->BytesPerPixel == 4);
    
    SDL_LockSurface(src);
    SDL_LockSurface(dst);

    
    dst_line_ptr = ((unsigned char*)dst->pixels) + dstrect->y * dst->pitch + dstrect->x * 4;
    src_line_ptr = ((unsigned char*)src->pixels) + srcrect->y * src->pitch + srcrect->x * 4;
    height = srcrect->h;
    
    
    while ( height-- ) {
	src_ptr = src_line_ptr;
	dst_ptr = dst_line_ptr;
	
	width = srcrect->w;
	for ( int c = width; c; --c ) {
	    
	    getCompose( dst_ptr[0],dst_ptr[1],dst_ptr[2],dst_ptr[3],
	    		 src_ptr[0],src_ptr[1],src_ptr[2],src_ptr[3],
	    		 dst_ptr[0],dst_ptr[1],dst_ptr[2],dst_ptr[3]);
	    
	    dst_ptr += 4;
	    src_ptr += 4;
	}
	src_line_ptr += src->pitch;
	dst_line_ptr += dst->pitch;
    }
    SDL_UnlockSurface(src);
    SDL_UnlockSurface(dst);
}

void SDL_ComposeSurface(SDL_Surface *src, SDL_Rect *vsrcrect, SDL_Surface *dst, SDL_Rect *dstrect)
{
    SDL_Rect tmpsrcrect = (*vsrcrect);
    SDL_Rect * srcrect = &tmpsrcrect;
    
    if (!dstrect->w) return;
    if (!dstrect->h) return;
    
    if (dstrect->x < 0) {
	if (dstrect->w <= (-dstrect->x)) return;
	dstrect->w += dstrect->x;
	srcrect->x -= dstrect->x;
	dstrect->x = 0;
    }
    
    if (dstrect->y < 0) {
	if (dstrect->h <= (-dstrect->y)) return;
	dstrect->w += dstrect->x;
	srcrect->x -= dstrect->x;
	dstrect->x = 0;
    }
    if (srcrect->x < 0) {
	if (dstrect->w <= (-srcrect->x)) return;
	dstrect->w += srcrect->x;
	dstrect->x -= srcrect->x;
	srcrect->x = 0;
    }
    if (srcrect->y < 0) {
	if (dstrect->h <= (-srcrect->y)) return;
	dstrect->h += srcrect->y;
	dstrect->y -= srcrect->y;
	srcrect->y = 0;
    }
    
    if (srcrect->x + dstrect->w > src->w) {
	int trop = srcrect->x + dstrect->w - src->w;
	if (trop >= dstrect->w) return;
	dstrect->w -= trop;
    }
    
    
    if (srcrect->y + dstrect->h > src->h) {
	int trop = srcrect->y + dstrect->h - src->h;
	if (trop >= dstrect->h) return;
	dstrect->h -= trop;
    }
		
    
    int use_global_alpha = 0;
    if ((dst->format->BitsPerPixel != 32)||(dst->format->Amask ==0)) {
	// No alpha in dst... just a blit then !
	SDL_BlitSurface(src,srcrect,dst,dstrect);
	return;
    }
    
    if (src->format->Amask == 0) {
	// No alpha channel in src...
	if (src->format->alpha == 255) {
	    // Put an opaque image ? Just a blit, then...
	    SDL_BlitSurface(src,srcrect,dst,dstrect);
	    return;
	} else if (src->format->alpha == 0) {
	    // Dumb ...
	    return;
	} else {
	    calcTable();
	    Compose_RGBA_RGB(src,srcrect,dst,dstrect,src->format->alpha);
	}
    } else {
	calcTable();
	Compose_RGBA_RGBA(src,srcrect,dst,dstrect);
    }
}

void SDL_ComposeBox(SDL_Surface *dst, SDL_Rect *dstrect, Uint32 color)
{
    unsigned char * dst_ptr, * dst_line_ptr;
    int width, height;
    int r,g,b,a;
    assert(dst->format->BytesPerPixel == 4);
    
    calcTable();
    
    SDL_LockSurface(dst);
    
    a = (color & 0x000000ff);
    r = (color & 0x0000ff00) >> 8;
    g = (color & 0x00ff0000) >> 16;
    b = (color & 0xff000000) >> 24;
    dst_line_ptr = ((unsigned char*)dst->pixels) + dstrect->y * dst->pitch + dstrect->x * 4;
    height = dstrect->h;
    
    
    while ( height-- ) {
	dst_ptr = dst_line_ptr;
	
	width = dstrect->w;
	for ( int c = width; c; --c ) {
	    getCompose(dst_ptr[0],dst_ptr[1],dst_ptr[2],dst_ptr[3],
	    		 r,g,b,a,
			 dst_ptr[0],dst_ptr[1],dst_ptr[2],dst_ptr[3]);
	    
	    dst_ptr += 4;
	}
	dst_line_ptr += dst->pitch;
    }
    SDL_UnlockSurface(dst);
}

void SDL_AdjustAlpha(SDL_Surface *dst, SDL_Rect *dstrect, int level)
{
    unsigned char * dst_ptr, * dst_line_ptr;
    int width, height;
    int r,g,b,a;
    assert(dst->format->BytesPerPixel == 4);
    
    SDL_LockSurface(dst);
    
    dst_line_ptr = ((unsigned char*)dst->pixels) + dstrect->y * dst->pitch + dstrect->x * 4;
    height = dstrect->h;
    
    level = (65536 * level) / 255;
    while ( height-- ) {
	dst_ptr = dst_line_ptr;
	
	width = dstrect->w;
	for ( int c = width; c; --c ) {
	    dst_ptr[3] = ((int)dst_ptr[3] * level) >> 16;
	    
	    dst_ptr += 4;
	}
	dst_line_ptr += dst->pitch;
    }
    SDL_UnlockSurface(dst);
}

static int alpha(int a1,int a2)
{
    return a1+a2-(a1*a2 / 255);
}

static int mc1(int a1,int a2,int alpha)
{
    return ((a1 * 255)+a1*a2) / alpha;
}

static int mc2(int a1,int a2,int alpha)
{
    return (a2 * 255)/alpha;
}

static void calcFactors(int a1,int a2,
                        unsigned char & ra, int & m1, int & m2)
{
    if (a1 == 0) {
        ra = a2;
        m1 = 0;
        m2 = 255 * a2;
        return;
    }
    if (a2 == 0) {
        ra = a1;
        m1 = 255 * a1;
        m2 = 0;
        return;
    }
    ra = alpha(a1,a2);
    if (ra == 0) {
        m1 = 0;
        m2 = 0;
        return;
    }
    m1 = (255 * a1 - a1 * a2);
    m2 = 255 * a2;
}

static void calcCompose(unsigned char r1,unsigned char g1,unsigned char b1,unsigned char a1,
                        unsigned char r2,unsigned char g2,unsigned char b2,unsigned char a2,
                        unsigned char & rr,unsigned char & rg,unsigned char & rb,unsigned char & ra )
{
    int m1,m2;
    calcFactors(a1,a2,ra,m1,m2);
    
    rr = ( r1 * m1 + r2 * m2 ) / (255 * ra);
    rg = ( g1 * m1 + g2 * m2 ) / (255 * ra);
    rb = ( b1 * m1 + b2 * m2 ) / (255 * ra);         
}

static void calcTable()
{
    if (tableok) return;
    tableok = true;
    for(int i = 0; i <= 255; ++i) {  
      for(int j = 0; j <= 255; ++j) {
        unsigned char ra;
        int m1,m2;
        
        calcFactors(i,j,ra,m1,m2);
        tbl[i * 1024 + j *4] = ra;
        if (ra > 0) {
          m1 = (128 * m1) / (255 * ra);
          m2 = (128 * m2) / (255 * ra);
        } else {
          m1 = 0;
          m2 = 0;
        }

        tbl[i * 1024 + j *4 + 1] = m1;
        tbl[i * 1024 + j *4 + 2] = m2;
      }
    }
}



int test()
{
    int r1,g1,b1,a1,r2,g2,b2,a2;
    unsigned char r,g,b,a;
    r1 = 0;
    g1 = 255;
    b1 = 0;
    a1 = 16;
    
    r2 = 255;
    g2 = 0;
    b2 = 128;
    a2 = 64;
    
    
    calcTable();
    printf("(%d,%d,%d %d) x (%d,%d,%d %d) =",r1,g1,b1,a1,r2,g2,b2,a2);
    calcCompose(r1,g1,b1,a1,r2,g2,b2,a2,r,g,b,a);
    printf("(%d,%d,%d %d)\n",r,g,b,a);
    getCompose(r1,g1,b1,a1,r2,g2,b2,a2,r,g,b,a);
    printf("(%d,%d,%d %d)\n",r,g,b,a);
}
/*
    int count, loop;
    unsigned char * src1, * src2;
    count = 5000;
    src1 = (unsigned char *)malloc(sizeof(unsigned char) * 4 * count);
    src2 = (unsigned char *)malloc(sizeof(unsigned char) * 4 * count);
    
    for(int i = 0; i < 4 * count; ++i ) {
        src1[i] = rand();
        src2[i] = rand();
    }
    
    printf("start...\n");
    for(loop = 0; loop < 10000; ++loop) {
      int idx = 0;
      for(int i = 0; i < count; ++i) {
          getCompose(src1[idx],src1[idx + 1],src1[idx+2],src1[idx + 3],
                      src2[idx],src2[idx + 1],src2[idx+2],src2[idx + 3],
                      src1[idx],src1[idx + 1],src1[idx+2],src1[idx + 3]);
          idx += 4;
      }
    }
    printf("stop...\n");
}*/
