#include "Task.h"
#include "jsreflect.h"

TaskNotification::TaskNotification(InternalTask * vfrom,InternalTask * vto) 
{
    from = vfrom;
    to = vto;
}

TaskNotification::~TaskNotification()
{
}

TaskQueue::TaskQueue()
{
    tasks = new std::list<InternalTask*>* [4];
    for(int  i = 0; i < 4 ; ++i) {
	tasks[i] = new std::list<InternalTask*> ();
    }
    signaled = false;
}

TaskQueue::~TaskQueue()
{
    // never...
}

InternalTask * TaskQueue::nextReady() 
{
    signaled = false;
    
    if (tasks[TASK_READY]->empty()) {
	
	return 0;
    } else {
	InternalTask * ret = 0;
	int bestp;
	
	for(typeof(tasks[TASK_READY]->begin()) it = tasks[TASK_READY]->begin(); it != tasks[TASK_READY]->end(); ++it) {
	    if ((!ret)||((*it)->priority < bestp)) {
		ret = (*it);
		bestp = (*it)->priority;
	
	    }
	}
	
	return ret;
    }
}


bool TaskQueue::step()
{
    InternalTask * torun;
    InternalTask::lock();
    if (torun = nextReady()) {
	taskRun(torun);
    }
    InternalTask::unlock();
    return torun != 0;
}

void TaskQueue::taskRun(InternalTask * torun) 
{
    TaskNotification * tn;
    setState(torun,TASK_RUNNING);
    
    if (!torun->pendingNotification.empty()) {
        typeof( torun->pendingNotification.begin()) it =  torun->pendingNotification.begin();
        
        tn = it->second;
        tn->to->pendingNotification.erase(tn->from);
        tn->from->emitedNotification.erase(tn->to);
    } else {
        tn = 0;
    }
    //InternalTask::unlock();
    torun->run(tn);
    //InternalTask::lock();
    if (torun->pendingNotification.empty()) {
	setState(torun,TASK_WAITING);
    } else {
	setState(torun,TASK_READY);
    }
    if (torun->suicide()) {
	delete(torun);
    }
}

void TaskQueue::setState(InternalTask * t,int state)
{
    if (state == t->state) {
	return;
    }
    if (t->state != -1) {
	tasks[t->state]->remove(t);
    }
    t->state = state;
    if (state != -1) {
	tasks[state]->push_back(t);
	
	if (state == TASK_READY) {
	    if (!signaled) {
		wakeup();
	    }
	    signaled = true;
	}
    }
}

InternalTask::InternalTask(TaskQueue * tq)
{
    jsrlog("Created %#x\n",this);
    state = -1;
    priority = 0;
    taskQueue = tq;
    lock();
    tq->setState(this,TASK_STOPPED);
    unlock();
}

InternalTask::~InternalTask()
{
    jsrlog("Deleted %#x, state = %d\n",this,state);
    lock();
    if (state == 1) {
	jsrlog("ERROR : deleting running task!\n");
    }
    for(typeof(waiting.begin()) it = waiting.begin(); it != waiting.end(); ++it) {
	(*it)->waitedFor.erase(this);
    }
    for(typeof(waitedFor.begin()) it = waitedFor.begin(); it != waitedFor.end(); ++it) {
	(*it)->waiting.erase(this);
    }
    
    for(typeof(pendingNotification.begin()) it = pendingNotification.begin() ; it != pendingNotification.end(); ++it) {
        TaskNotification * tnf = it->second;
        if (tnf->from != this) {
            tnf->from->emitedNotification.erase(this);
            delete it->second;
        }
    }

    for(typeof(emitedNotification.begin()) it = emitedNotification.begin() ; it != emitedNotification.end(); ++it) {
        TaskNotification * tnf = it->second;
        if (tnf->to != this) {
            tnf->to->pendingNotification.erase(this);
        }
        delete it->second;
    }
    
    taskQueue->setState(this,-1);
    unlock();
}

void InternalTask::setPriority(int p)
{
    InternalTask::lock();
    priority = p;
    InternalTask::unlock();
}

void InternalTask::waitNotification(InternalTask * t)
{
    waitedFor.insert(t);
    t->waiting.insert(this);
    if (state != TASK_RUNNING) {
        taskQueue->setState(this,TASK_WAITING);
    }
}

void InternalTask::unwaitNotification(InternalTask * t)
{
    waitedFor.erase(t);
    t->waiting.erase(this);
    // Remove any pending notification from t
    if (pendingNotification.find(t) != pendingNotification.end()) {
	TaskNotification * tn;
	tn = pendingNotification[t];
	pendingNotification.erase(t);
	t->emitedNotification.erase(this);
	delete(tn);
    }
}

void InternalTask::emitNotification()
{
    for(typeof(waiting.begin()) it = waiting.begin(); it != waiting.end(); ++it) {
	// If t is not already signalled...
	if (emitedNotification.find(this) == emitedNotification.end()) {
	    InternalTask * t = (*it);
	    
	    createNotification(t);
	}
    }
}

void InternalTask::createNotification(InternalTask * t)
{
    TaskNotification *tnf = new TaskNotification(this,t);
    emitedNotification[t] =tnf;
    t->pendingNotification[this] = tnf;
    
    if (t->state != TASK_RUNNING) {
	t->taskQueue->setState(t,TASK_READY);
    }
}

void InternalTask::start()
{
    taskQueue->setState(this,TASK_READY);
}

void InternalTask::autoSignal()
{
    createNotification(this);
}

void InternalTask::run(TaskNotification * tn)
{
}

bool InternalTask::pendingSignal()
{
    return state == TASK_READY;
}

bool InternalTask::suicide()
{
    return false;
}

bool InternalTask::isRunning()
{
    return state == TASK_RUNNING;
}

static SDL_mutex * taskMutex;

void InternalTask::lock()
{
    SDL_mutexP(taskMutex);
}

void InternalTask::unlock()
{
    SDL_mutexV(taskMutex);
}

void InternalTask::condWait(SDL_cond * cond)
{
    SDL_CondWait(cond,taskMutex);
}

class CondTaskQueue : public TaskQueue {
    SDL_cond * cond;
    
    virtual void wakeup() {
	SDL_CondBroadcast(cond);
    }
    
public:
    CondTaskQueue() : TaskQueue() {
	cond = SDL_CreateCond();
    }
    
    virtual ~CondTaskQueue() {
    }
    
    void run() {
	InternalTask * torun;
	InternalTask::lock();
	while (!InternalTask::exitFlag()) {
	    while((!(torun = nextReady()))&&(!InternalTask::exitFlag())) {
		InternalTask::condWait(cond);
	    }
	    
	    if (!InternalTask::exitFlag()) {
		taskRun(torun);
	    }
	}
	InternalTask::unlock();
    }
};

class SyncTaskQueue : public TaskQueue {
    int fd;
    
    virtual void wakeup() {
	write(fd,&fd,1);
    }
public:
    SyncTaskQueue(int vfd) {
	fd = vfd;
    }
};

static int startAsyncThread(void * v)
{
    CondTaskQueue *ctq = (CondTaskQueue *)v;
    ctq->run();
    return 0;
}

TaskQueue *  InternalTask::newAsyncTaskQueue(int nb)
{
    CondTaskQueue * ctq = new CondTaskQueue();
    for(int i = 0 ; i < nb; ++i) {
	SDL_CreateThread(&startAsyncThread,(void*)ctq);
    }
    return ctq;
}

void InternalTask::startAsyncThreads(int nb)
{
    taskMutex = SDL_CreateMutex();
    asyncTaskQueue = newAsyncTaskQueue(nb);
}

void InternalTask::initSyncThread(int fd)
{
    syncTaskQueue = new SyncTaskQueue(fd);
}

TaskQueue * InternalTask::asyncTaskQueue;
TaskQueue * InternalTask::syncTaskQueue;


static bool staticExitFlag = false;

bool InternalTask::exitFlag() {
    return staticExitFlag;
}



