#include <SDL.h>
#include <SDL_image.h>

#include "jsreflect.h"
#include "XmlParser.h"
#include "Graphs.h"
#include "Application.h"
#include "Widget.h"
#include "GridWidget.h"
#include "ImageWidget.h"
#include "Layout.h"
#include "Style.h"
#include "JWidgetClass.h"
#include <string>

class LayoutCell_XmlItem : public XmlJsObject {
public:
    LayoutCell_XmlItem() : XmlJsObject()
    {
    }
    
    virtual void addAttribute(const std::string & name,const std::string & value)
    {
	if (name == "width") {
	    ((LayoutCell*)object)->setStrWidth(value);
	} else if (name == "leftBorder") {
	    ((LayoutCell*)object)->setStrLeftBorder(value);
	} else if (name == "rightBorder") {
	    ((LayoutCell*)object)->setStrRightBorder(value);
	} else {
	    XmlJsObject::addAttribute(name,value);
	}
    }
};
    

class LayoutLine_XmlItem : public XmlJsObject {
public:
    LayoutLine_XmlItem() : XmlJsObject()
    {
    }

    virtual void addAttribute(const std::string & name,const std::string & value)
    {
	if (name == "height") {
	    ((LayoutLine*)object)->setStrHeight(value);
	} else if (name == "topBorder") {
	    ((LayoutLine*)object)->setStrTopBorder(value);
	} else if (name == "bottomBorder") {
	    ((LayoutLine*)object)->setStrBottomBorder(value);
	} else {
	    XmlJsObject::addAttribute(name,value);
	}
    }
    
    
    virtual XmlParsableItem * newChild(const std::string & name)
    {
	if (name == "cell") {
	    LayoutCell * l = new LayoutCell();
	    l->definition = parser->getLocation();
	    ((LayoutLine*)object)->addCell(l);
	    XmlJsObject * xmli = new LayoutCell_XmlItem();
	    xmli->setObject(l);
	    return xmli;
	}
	return XmlJsObject::newChild(name);
    }
};

class Layout_XmlItem : public XmlJsObject {
public:
    Layout_XmlItem() : XmlJsObject()
    {
    }
    
    virtual XmlParsableItem * newChild(const std::string & name)
    {
	if (name == "line") {
	    LayoutLine * l = new LayoutLine();
	    l->definition = parser->getLocation();
	    ((GridLayout*)object)->addLine(l);
	    LayoutLine_XmlItem * xmli = new LayoutLine_XmlItem();
	    xmli->setObject(l);
	    return xmli;
	}
	return XmlJsObject::newChild(name);
    }
    
    virtual void done() {
	((Layout*)object)->update();
    }
};

class WidgetFather_XmlItem : public XmlJsObject {
    Widget * firstLoaded;
public:
    WidgetFather_XmlItem() : XmlJsObject()
    {
	firstLoaded = 0;
    }
    virtual ~WidgetFather_XmlItem() {
    }
    Widget * getFirstLoaded() const {
	return firstLoaded;
    }
    
    virtual XmlParsableItem * newChild(const std::string & name);
};

class Widget_XmlItem : public WidgetFather_XmlItem {
public:
    Widget_XmlItem() : WidgetFather_XmlItem()
    {
    }
    
    virtual XmlParsableItem * newChild(const std::string & name);
};

class XmlWidget_XmlItem : public Widget_XmlItem {
    Widget * father;
    
    void ensureObject() {
	if (!object) {
	    object = new Widget();
	    ((Widget*)object)->setFather(father);
	}
    }
public:
    XmlWidget_XmlItem(Widget * f) : Widget_XmlItem() {
	father = f;
    }
    
    virtual XmlParsableItem * newChild(const std::string & name) {
	ensureObject();
	return Widget_XmlItem::newChild(name);
    }
    
    virtual void addAttribute(const std::string & name,const std::string & value)
    {
	if (name == "source") {
	    if (object != 0) {
		fprintf(stderr,"XmlWidget : ignoring source (too late)\n");
	    } else {
		object = Application::loadXmlWidget(value.c_str(),father);
	    }
	    return;
	}
	ensureObject();
	Widget_XmlItem::addAttribute(name,value);
    }
    
    virtual void done() {
	ensureObject();
	Widget_XmlItem::done();
	// Fixme : evaluate loaded on object...
	((Widget*)object)->evaluate("onloaded()",JSReflection::JSSourceInfo("internal : onloaded()",0));
    }
};

XmlParsableItem * WidgetFather_XmlItem::newChild(const std::string & name)
{
    /*if (name == "xmlWidget") {
	XmlWidget_XmlItem * xmli = new XmlWidget_XmlItem(object);
	return xmli;
    }*/
    typeof(WidgetClass::classes.find(name)) it = WidgetClass::classes.find(name);
    if (it != WidgetClass::classes.end()) {
	WidgetClass * wc = it->second;
	Widget * v = wc->createInstance((object?(Widget*)object:0));
	if (v) {
	    if (!firstLoaded) firstLoaded = v;
	    Widget_XmlItem * xmli = new Widget_XmlItem();
	    xmli->setObject(v);
	    return xmli;
	} else {
	    jsrlog("failed to create instance of %s\n",name.c_str());
	}
    }
    
    return XmlJsObject::newChild(name);
}

class StyleFilter_XmlItem : public XmlParsableItem {
    Style * style;
    std::string substyle_name;
    bool substyle_value;
    bool hasname;
    bool hasvalue;
public:
    StyleFilter_XmlItem(Style * s) : XmlParsableItem(0) {
    	style = s;
	substyle_value = true;
	hasname = false;
	hasvalue = false;
    }
    
    virtual void addAttribute(const std::string & name,const std::string & value)
    {
	if (name == "substyle") {
	    substyle_name = value;
	    hasname = true;
	    return;
	}
	if (name == "value") {
	    substyle_value = (value != "0");
	    hasvalue = true;
	    return;
	}
	return XmlParsableItem::addAttribute(name,value);
    }
    
    virtual void done()
    {
	if (hasname) {
	    style->filters[substyle_name] = substyle_value;
	} else {
	    jsrlog("style filter has no name...\n");
	}
    }
};

class StyleValue_XmlItem : public XmlParsableItem {
    Style * style;
    std::string substyle_name;
    std::string substyle_value;
    bool hasname;
    bool hasvalue;
public:
    StyleValue_XmlItem(Style * s) : XmlParsableItem(0) {
    	style = s;
	substyle_value = true;
	hasname = false;
	hasvalue = false;
    }
    
    virtual void addAttribute(const std::string & name,const std::string & value)
    {
	if (name == "name") {
	    substyle_name = value;
	    hasname = true;
	    return;
	}
	if (name == "value") {
	    substyle_value = value;
	    hasvalue = true;
	    return;
	}
	return XmlParsableItem::addAttribute(name,value);
    }
    
    virtual void done()
    {
	if (hasname && hasvalue) {
	    style->values[substyle_name] = substyle_value;
	} else {
	    jsrlog("style assign has no name or value...\n");
	}
    }
};

class Style_XmlItem : public XmlParsableItem {
    Style * style;
    bool added;
public:
    Style_XmlItem() : XmlParsableItem(0)
    {
	style = new Style();
	added = false;
    }
    
    virtual ~Style_XmlItem()
    {
    }
    
    virtual void addAttribute(const std::string & name,const std::string & value)
    {
	if (name == "name") {
	    if (!added) {
		added = true;
		jsrlog("readed new log : %s\n",name.c_str());
		Style::allStyles[value].push_back(style);
	    }
	    return;
	} else {
	    XmlParsableItem::addAttribute(name,value);
	}
    }
    
    XmlParsableItem * newChild(const std::string & name) 
    {
	if (name == "filter") {
	    return new StyleFilter_XmlItem(style);
	}
	if (name == "set") {
	    return new StyleValue_XmlItem(style);
	}
	return XmlParsableItem::newChild(name);
    }
    
};

class StyleSheet_XmlItem : public XmlParsableItem {
public:
    StyleSheet_XmlItem() : XmlParsableItem(0)
    {}
    
    XmlParsableItem * newChild(const std::string & name) 
    {
	if (name == "style") {
	    return new Style_XmlItem();
	}
	return XmlParsableItem::newChild(name);
    }
};


class AnyRoot_XmlItem : public WidgetFather_XmlItem {
public:
    AnyRoot_XmlItem(Widget * nfather) : WidgetFather_XmlItem()
    {
	setObject(nfather);
    }
    
    virtual ~AnyRoot_XmlItem()
    {}
    
    XmlParsableItem * newChild(const std::string & name)
    {
	if (name == "global") {
	    XmlJsObject * xjo = new XmlJsObject();
	    xjo->setObject(Application::current->getJsContext()->GetGlobal());
	    return xjo;
	}
	if (name == "styles") {
	    return new StyleSheet_XmlItem();
	}
	return WidgetFather_XmlItem::newChild(name);
    }
};

XmlParsableItem * Widget_XmlItem::newChild(const std::string & name)
{
    if (name == "gridlayout") {
	Layout * l = new GridLayout();
	((Widget*)object)->setLayout(l);
	Layout_XmlItem * xmli = new Layout_XmlItem();
	xmli->setObject(l);
	return xmli;
    }
    if (name == "fulllayout") {
	Layout * l = new FullLayout();
	((Widget*)object)->setLayout(l);
	XmlJsObject * xmli = new XmlJsObject();
	xmli->setObject(l);
	return xmli;
    }
    if (name == "input") {
	InputEventHandler * iev = new InputEventHandler((Widget*)object);
	XmlJsObject * xmli= new XmlJsObject();
	xmli->setObject(iev);
	return xmli;
    }
    if (name == "xmlWidget") {
	XmlWidget_XmlItem * xmli = new XmlWidget_XmlItem((Widget*)object);
	return xmli;
    }
    return WidgetFather_XmlItem::newChild(name);
}

bool Application::loadXml(const char * xmlFileName, Widget * w )
{
    XmlParser parser;
    
    AnyRoot_XmlItem * root = new AnyRoot_XmlItem(w);

    parser.pushItem(root);
    if (!parser.parseDocument(xmlFileName)) {
	fprintf(stderr,"parsing of %s failed\n",xmlFileName);
	return false;
    }
    return true;
}

Widget * Application::loadXmlWidget(const char * xmlFileName,Widget * father) 
{
    XmlParser parser;
    
    WidgetFather_XmlItem * root = new WidgetFather_XmlItem();
    root->setAutoKill(false);
    root->setObject(father);
    // root will be deleted when parser will get destroyed !
    parser.pushItem(root);
    if (!parser.parseDocument(xmlFileName)) {
	fprintf(stderr,"parsing of %s failed\n",xmlFileName);
	return 0;
    }
    root->setAutoKill(true);
    Widget * w = root->getFirstLoaded();
    return w;
}

bool Widget::loadXml(const char * xmlFileName)
{
    return Application::loadXml(xmlFileName,this);
}

static bool loadXml(const char * xmlFileName)
{
    return Application::loadXml(xmlFileName);
}

JS_NOCONTEXT_GLOBAL_FUNCTION_1(loadXml,bool,"%b",0,JSReflection::JS_String,"%s",0);
