#include <SDL.h>
#include <SDL_thread.h>
#include <SDL_image.h>

#include <stddef.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>

#include <stdexcept>
#include <deque>
using namespace std;

#include "Graphs.h"
#include "Widget.h"
#include "Application.h"
#include "lirc.h"



LircListener::LircListener() : Selectable(), JSReflection::Object(Application::getJsContext())
{
}

LircListener::~LircListener()
{
    if (getFd() != -1) {
	close(getFd());
	setFd(-1);
    }
}

bool LircListener::isOpen() const
{
    return getFd() != -1;
}

void LircListener::parseLine(const std::string & line)
{
    long long n;
    unsigned t1,t2;
    t1 = line.find(' ');
    t1 = line.find(' ', t1 + 1);
    t2 = line.find(' ', t1 + 1);
    std::string symb = "LIRC:"+line.substr(t1 + 1,t2 - t1 - 1);
    n = Application::current->now();
    jsrlog("got symb %s, last symb is %s %d %d\n",symb.c_str(),lastsymb.c_str(), n - lastsymbtime, n - firstsymbtime);
    if ((symb != lastsymb) || (n - lastsymbtime > 500)) {
	lastsymb = symb;
	firstsymbtime = n;
	lastsymbtime = n;
    	Application::current->inputEvent(symb);
    } else { // Repetition
	if (n - firstsymbtime > 1000) {
	    lastsymbtime = n;
	    Application::current->inputEvent(symb);
	} else {
	    lastsymbtime = n;
	    jsrlog("ignore\n");
	}
    }
}

void LircListener::parseReceived()
{
    unsigned int pos;
    while ((pos = received.find('\n')) != std::string::npos) {
    	std::string tmp = received.substr(0, pos - 1);
	received = received.substr(pos+1);
	parseLine(tmp);
    }
}

void LircListener::doRead()
{
    char tmp[4096];
    int ret;
    do {
	ret = read(getFd(),tmp,4096);
    } while ((ret == -1) && (errno == EINTR));
    
    if ((ret == 0) || (ret == -1)) {
	if (ret == -1) {
	    perror("lirc");
	}
	close(getFd());
	setFd(-1);
	if (received.length() > 0) {
	    parseLine(received);
	}
	received = "";
	return;
    }
    received += std::string(tmp,ret);
    parseReceived();
}

void LircListener::open(const std::string & filename)
{
    struct sockaddr_un name;
    
    int sock;
    size_t size;
    
    
    /* Create the socket. */
    sock = socket (PF_LOCAL, SOCK_STREAM, 0);
    if (sock < 0) {
       	perror ("socket");
	return;
    }
    
    /* Bind a name to the socket. */
    name.sun_family = AF_LOCAL;
    strncpy (name.sun_path, filename.c_str(), sizeof (name.sun_path));
    name.sun_path[sizeof (name.sun_path) - 1] = '\0';
    
    /* The size of the address is
      the offset of the start of the filename,
      plus its length,
      plus one for the terminating null byte.
      Alternatively you can just do:
      size = SUN_LEN (&name);
    */
    size = (offsetof (struct sockaddr_un, sun_path)
	   + strlen (name.sun_path) + 1);
    if (connect(sock,(struct sockaddr*) &name,size) < 0){
	perror("connect");
	return;
    }
    
    if (getFd() != -1) {
	close(getFd());
    }
    setFd(sock);
}

LircListener * newLircListener()
{
    return new LircListener();
}

JS_EXPORT_CLASS(LircListener,JSReflection::Object)
JS_NOCONTEXT_GLOBAL_FUNCTION_0(newLircListener,LircListener*,"%x",LircListener::ClassInfo);
JS_NOCONTEXT_OBJECT_VOID_FUNCTION_1(LircListener,open,JSReflection::JS_String,"%s",0);
JS_RO_PROPERTY(LircListener,isOpen,isOpen,bool,"%b",0);

