#ifndef XMLPARSER_H
#define XMLPARSER_H 1

#include <string>
#include <list>

class XmlParser;

class XmlParsableItem {
protected:
    JSReflection::Context * context;
    XmlParser * parser;
    bool autokill;
public:
    XmlParsableItem(JSReflection::Context * context = 0);
    virtual ~XmlParsableItem();
    
    void setParser(XmlParser * p); 
    
    virtual void init();
    virtual void addAttribute(const std::string & name,const std::string & value);
    virtual void addText(const std::string & text);
    virtual XmlParsableItem * newChild(const std::string & name);
    virtual void done();	/// Called when child is terminated

    void setAutoKill(bool b);
    bool getAutoKill() const;
    
    

    JSReflection::Context * getContext() const;
};


class XmlJsScript : public XmlParsableItem {
protected:
    bool scopeProtected;
    JSReflection::Object * object;
    JSObject * scope;
    JSReflection::JSSourceInfo jssi;
    std::string script;
public:
    XmlJsScript(JSReflection::Context * c = 0);
    virtual ~XmlJsScript();
    
    void setObject(JSReflection::Object * object);
    
    virtual void init();
    virtual void addText(const std::string & text);
    virtual void addAttribute(const std::string & name,const std::string & value);
    virtual XmlParsableItem * newChild(const std::string & name);
    virtual void done();
};

class XmlJsObject : public XmlParsableItem {
protected:
    JSReflection::Object * object;
public:
    XmlJsObject(JSReflection::Context * c = 0);
    virtual ~XmlJsObject();
    
    void setObject(JSReflection::Object * object);
    
    virtual void addAttribute(const std::string & name,const std::string & value);
    virtual XmlParsableItem * newChild(const std::string & name);
    /*virtual void addText(const std::string & text);
    
    virtual void done();*/
};

class XmlParser {
    std::list<XmlParsableItem*> stack;
    std::string curFileName;
    void * ownstruct;
    void * xmlSimpleRoot;
    void * xmlctxt;
public:
    
    XmlParsableItem * current() const;
    void pushItem(XmlParsableItem * newItem);
    void popItem();

    XmlParser();
    virtual ~XmlParser();
    
    bool parseDocument(const std::string & filename);
    bool parseString(const std::string & content, const std::string & filename);
   
    void start(const std::string & filename,JSReflection::Context * c);
    void parse(const char * t,int length,std::list<JSReflection::Object*> & add); // Return new objects.
    void done();
    
    JSReflection::JSSourceInfo getLocation() const;
};


#endif
