/***************************************************************************
*   Copyright (C) 2004 by ludovic p                                       *
*   pludov@nnx.com                                                        *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <stdlib.h>

#include "jsreflect.h"


using namespace JSReflection;

JS_EXPORT_CLASS( Closure, Object );

Closure::Closure( Context * c ) : Object( c )
{}

Closure::~Closure()
{}

void Closure::RelayedLost()
{
    delete(this);
}

void Closure::RelayerLost()
{
    delete(this);
}

void Closure::HandleEvent(Event * e)
{
    
    if (RelayEventTo) {
	int i;
	i = 0;
	while (RelayEventTo[i]) {
	    Call(RelayEventTo[i++], e);
	}
    }
}

JS_EXPORT_CLASS( JSClosure, Closure )

JSClosure::JSClosure( Context * c, JSFunction * call, uintN argc, jsval * argv ) : Closure( c )
{
    if (argc) {
        Args = ( jsval* ) malloc( sizeof( jsval ) * argc );
        ArgCount = argc;
        for ( int i = 0 ;i < ArgCount; i++ )
        {
            Args[ i ] = argv[ i ];
            JS_AddRoot(ReflectionContext->GetJSContext(), Args + i);
        }
    } else {
        ArgCount = 0;
    }
    Function = call;
    
}

JSClosure::~JSClosure()
{
    if (ArgCount) {
        for ( int i = 0 ;i < ArgCount; i++ )
        {
            JS_RemoveRoot(ReflectionContext->GetJSContext(), Args + i);
        }
    }
}

void JSClosure::Call( Object * o, Object * e )
{
    jsval parameters[ArgCount + 1];
    jsval ret;
    
    
    parameters[0] = OBJECT_TO_JSVAL(e->GetJSObject(true));
    
    for (int i = 0; i < ArgCount; i++) {
        parameters[i+1] = Args[i+1];
    }
    
    if (!JS_CallFunction(ReflectionContext->GetJSContext(), o->GetJSObject(true), Function, ArgCount + 1, parameters, &ret)) {
	fprintf(stderr,"Call function failed\n");
    }
}

void JSClosure::Call( Object * o, jsval arg)
{
    jsval parameters[ArgCount + 1];
    jsval ret;
    
    
    parameters[0] = arg;
    
    for (int i = 0; i < ArgCount; i++) {
        parameters[i+1] = Args[i+1];
    }
    
    if (!JS_CallFunction(ReflectionContext->GetJSContext(), o->GetJSObject(true), Function, ArgCount + 1, parameters, &ret)) {
	fprintf(stderr,"Call function failed\n");
    }
    
}

static JSBool JS_Create_Closure( JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * rval )
{
    JSReflection::Context * c = JSReflection::Context::Find(cx);
    JSFunction * f;
    JSClosure * result;
    
    if (!c) { return JS_FALSE; }
    if (argc < 1) {
        return JS_FALSE;
    }
    f = JS_ValueToFunction(cx, argv[0]);
    if (!f) {
        return JS_FALSE;
    }
    result = new JSClosure(c, f, argc - 1, argv + 1);
    jsrlog("new closure at %#x\n",(void*)(Closure*)result);
    return c->ValueToJS(&result,rval, "%x", JSClosure::ClassInfo);
}

static JSReflection::FunctionInfo JS_Create_Closure_jsfunction 
                        (&JSReflection::Context::PublicFunctions, "Closure", &JS_Create_Closure, 1);
