/***************************************************************************
 *   Copyright (C) 2004 by ludovic p                                       *
 *   pludov@nnx.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <stdlib.h>

#include "jsreflect.h"

using namespace JSReflection;

JS_EXPORT_CLASS( EventFIFO, JSReflection::Object )
 
EventFIFO::EventFIFO(Context * c):Object(c)
{    
    First = 0;
    Last = 0;
}

EventFIFO::~EventFIFO()
{
    while(First) {
        delete(First);
    }
}
    
bool EventFIFO::ProceedEvent()
{
    bool uihandled;
    
    uihandled = false;
    
    while (First) {
	/*if (First->isUiEvent() ) {
	    if (uihandled) {
		return;
	    }
	    uihandled = true;
	}
	*/
	
	Event * current = First;
	Object * target;
	target = current->GetTarget();
	if (!target) {
	    current->Dispatch();
	    target = current->GetTarget();
	}
	
	if (target) {
	    target->HandleEvent(current);
	    current->Treated();
	}
	
	delete(current);
    }
    return (First != 0);
}

Event * EventFIFO::GetFirstEvent() const {
    return First;
}

Event * EventFIFO::GetLastEvent() const {
    return Last;
}

JS_EXPORT_CLASS( Event, JSReflection::Object )

Event::Event(EventFIFO * f, const char * uniqId):Object(f->GetContext())
{
    Father = f;
    EventId = uniqId;
    
    NextInFIFO = 0;
    PrevInFIFO = f->Last;
    if (f->Last) {
	f->Last->NextInFIFO = this;
    } else {
	f->First = this;
    }
    f->Last = this;
    
    Source = 0;
    NextInSource = 0;
    PrevInSource = 0;
    
    Target = 0;
    NextInTarget = 0;
    PrevInTarget = 0;
}
        
Event::~Event()
{
    // Remove from FIFO
    if (Father) {
	if (PrevInFIFO) {
	    PrevInFIFO->NextInFIFO = NextInFIFO;
	} else {
	    Father->First = NextInFIFO;
	}
	
	if (NextInFIFO) {
	    NextInFIFO->PrevInFIFO = PrevInFIFO;
	} else {
	    Father->Last = PrevInFIFO;
	}
    }
    
    // Disconnect from current source
    if (Source) {
        if (NextInSource) {
            NextInSource->PrevInSource = PrevInSource;
        } else {
            Source->LastSourcing = PrevInSource;
        }
    
        if (PrevInSource) {
            PrevInSource->NextInSource = NextInSource;   
        } else {
            Source->FirstSourcing = NextInSource;
        }
    }

    // Disconnect from current target
    if (Target) {
        if (NextInTarget) {
            NextInTarget->PrevInTarget = PrevInTarget;
        } else {
            Target->LastTargeting = PrevInTarget;
        }
    
        if (PrevInTarget) {
            PrevInTarget->NextInTarget = NextInTarget;   
        } else {
            Target->FirstTargeting = NextInTarget;
        }
    }
    
    // Free eventid
    if (EventId && (EventId[0] == '#')) {
	free((void*)EventId);
    }
}

void Event::SourceLost() {
    // Disconnect from current source
    if (NextInSource) {
	NextInSource->PrevInSource = PrevInSource;
    } else {
	Source->LastSourcing = PrevInSource;
    }

    if (PrevInSource) {
	PrevInSource->NextInSource = NextInSource;   
    } else {
	Source->FirstSourcing = NextInSource;
    }
    
    Source = 0;
}

void Event::TargetLost() {
    // Disconnect from current target
    if (NextInTarget) {
	NextInTarget->PrevInTarget = PrevInTarget;
    } else {
	Target->LastTargeting = PrevInTarget;
    }

    if (PrevInTarget) {
	PrevInTarget->NextInTarget = NextInTarget;   
    } else {
	Target->FirstTargeting = NextInTarget;
    }

    Target = 0;
}

void Event::SetRoute(Object * source, Object * target, Event * rightBefore)
{
    if (rightBefore==this) {
	rightBefore = NextInFIFO;
    }
    
    // Disconnect from current source
    if (Source) {
        if (NextInSource) {
            NextInSource->PrevInSource = PrevInSource;
        } else {
            Source->LastSourcing = PrevInSource;
        }
    
        if (PrevInSource) {
            PrevInSource->NextInSource = NextInSource;   
        } else {
            Source->FirstSourcing = NextInSource;
        }
    }

    // Disconnect from current target
    if (Target) {
        if (NextInTarget) {
            NextInTarget->PrevInTarget = PrevInTarget;
        } else {
            Target->LastTargeting = PrevInTarget;
        }
    
        if (PrevInTarget) {
            PrevInTarget->NextInTarget = NextInTarget;   
        } else {
            Target->FirstTargeting = NextInTarget;
        }
    }

    // Disconnect from current FIFO
    if (NextInFIFO) {
	NextInFIFO->PrevInFIFO = PrevInFIFO;
    } else {
	Father->Last = PrevInFIFO;
    }

    if (PrevInFIFO) {
	PrevInFIFO->NextInFIFO = NextInFIFO;   
    } else {
	Father->First = NextInFIFO;
    }
    
    Source = source;
    Target = target;

    if (rightBefore) {
	// Insert just before rightBefore
	PrevInFIFO = rightBefore->PrevInFIFO;
	NextInFIFO = rightBefore;
	rightBefore->PrevInFIFO = this;
	if (PrevInFIFO) {
	    PrevInFIFO->NextInFIFO = this;
	} else {
	    Father->First = this;
	}
    } else {
	// Insert at the end of the FIFO
	PrevInFIFO = Father->Last;
	NextInFIFO = 0;
	if (Father->Last) {
	    Father->Last->NextInFIFO = this;
	} else {
	    Father->First = this;
	}
	Father->Last = this;
    }
    
    if (Source) {
	Event * sourceBefore;
	sourceBefore = rightBefore;
	
	// Find the first object for the same source, at or after rightbefore
    	while (sourceBefore && (sourceBefore->Source != Source)) {
	    sourceBefore = sourceBefore->NextInFIFO;
    	}

	if (sourceBefore) {
	    PrevInSource = sourceBefore->PrevInSource;
	    NextInSource = sourceBefore;
	    sourceBefore->PrevInSource = this;
	    if (PrevInSource) {
		PrevInSource->NextInSource = this;
	    } else {
		Source->FirstSourcing = this;
	    }
	    
	} else {
	    //Insert at the end...
	    PrevInSource = Source->LastSourcing;
	    NextInSource = 0;
	    if (Source->LastSourcing) {
		Source->LastSourcing->NextInSource = this;
	    } else {
		Source->FirstSourcing = this;
	    }
	    Source->LastSourcing = this;
	}
    }
    
    if (Target) {
	Event * targetBefore;
	targetBefore = rightBefore;
	
	// Find the first object for the same Target, at or after rightbefore
    	while (targetBefore && (targetBefore->Target != Target)) {
	    targetBefore = targetBefore->NextInFIFO;
    	}

	if (targetBefore) {
	    PrevInTarget = targetBefore->PrevInTarget;
	    NextInTarget = targetBefore;
	    targetBefore->PrevInTarget = this;
	    if (PrevInTarget) {
		PrevInTarget->NextInTarget = this;
	    } else {
		Target->FirstTargeting = this;
	    }
	} else {
	    //Insert at the end...
	    PrevInTarget = Target->LastTargeting;
	    NextInTarget = 0;
	    if (Target->LastTargeting) {
		Target->LastTargeting->NextInTarget = this;
	    } else {
		Target->FirstTargeting = this;
	    }
	    Target->LastTargeting = this;
	}
    }
}

Object * Event::GetSource() const
{
    return Source;
}

Object * Event::GetTarget() const
{
    return Target;
}

Event * Event::GetNextInFifo() const{
    return NextInFIFO;
}

void Event::Dispatch() { }
void Event::Treated() { }

void Event::Debug() const
{
    printf("[%#x |   Event %#x : %s    | %#x]\n",PrevInFIFO, this, EventId, NextInFIFO);
}

bool Event::IsA(const char * id) const
{
    const char * cmpid;
    cmpid = EventId;
    if (!cmpid) return false;
    if (cmpid[0]=='#') cmpid++;
    
    return !strcmp(cmpid,id);
}

