/***************************************************************************
 *   Copyright (C) 2004 by ludovic p                                       *
 *   pludov@nnx.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define JS_INTERNAL_GETTER_CALLBACK(CLASSNAME,FUNCNAME,RESULT_TYPE,OUTPUTFORMAT,OUTPUTQUALIFIER)                \
static JSBool CLASSNAME ## _ ## FUNCNAME ## _callback (JSContext * cx, JSObject * obj, jsval id, jsval * vp)    \
{                                                                                                               \
    JSReflection::Context * c = JSReflection::Context::Find(cx);                                                \
    if (!c) { return JS_FALSE; }                                                                                \
    CLASSNAME * self;                                                                                           \
    RESULT_TYPE result;                                                                                         \
    if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; }               \
    result = self->FUNCNAME();                                                                                  \
    if (!c->ValueToJS((void*)&result,vp,OUTPUTFORMAT,(JSReflection::ClassInfo*)OUTPUTQUALIFIER)) { return JS_FALSE; }\
    return JS_TRUE;                                                                                             \
}

#define JS_INTERNAL_SETTER_CALLBACK(CLASSNAME,FUNCNAME,VALUE_TYPE,VALUEFORMAT,VALUEQUALIFIER)                   \
static JSBool CLASSNAME ## _ ## FUNCNAME ## _callback (JSContext * cx, JSObject * obj, jsval id, jsval * vp)    \
{                                                                                                               \
    JSReflection::Context * c = JSReflection::Context::Find(cx);                                                \
    if (!c) { return JS_FALSE; }                                                                                \
    CLASSNAME * self;                                                                                           \
    VALUE_TYPE value;                                                                                           \
    if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; }               \
    if (!c->ValueFromJS(vp,&value,VALUEFORMAT,VALUEQUALIFIER)) { return JS_FALSE; }                             \
    self->FUNCNAME(value);                                                                                      \
    return JS_TRUE;                                                                                             \
}

#define JS_RW_PROPERTY(CLASSNAME,PROPNAME,GETNAME,SETNAME,VALUE_TYPE,VALUEFORMAT,VALUEQUALIFIER)                \
    JS_INTERNAL_GETTER_CALLBACK(CLASSNAME,GETNAME,VALUE_TYPE,VALUEFORMAT,VALUEQUALIFIER)                        \
    JS_INTERNAL_SETTER_CALLBACK(CLASSNAME,SETNAME,VALUE_TYPE,VALUEFORMAT,VALUEQUALIFIER)                        \
    static JSReflection::PropertyInfo CLASSNAME ## _ ## PROPNAME ## _property                                   \
                        ( &CLASSNAME ## _ClassInfo::Properties, #PROPNAME,                                      \
                            &CLASSNAME ## _ ## GETNAME ## _callback,                                            \
                            &CLASSNAME ## _ ## SETNAME ## _callback);                                           \

#define JS_RO_PROPERTY(CLASSNAME,PROPNAME,GETNAME,VALUE_TYPE,VALUEFORMAT,VALUEQUALIFIER)                        \
    JS_INTERNAL_GETTER_CALLBACK(CLASSNAME,GETNAME,VALUE_TYPE,VALUEFORMAT,VALUEQUALIFIER)                        \
    static JSReflection::PropertyInfo CLASSNAME ## _ ## PROPNAME ## _property                                   \
                        ( &CLASSNAME ## _ClassInfo::Properties, #PROPNAME,                                      \
                            &CLASSNAME ## _ ## GETNAME ## _callback, 0);

#define JS_INTERNAL_VAR_GETTER(CLASSNAME,VARNAME,RESULT_TYPE,OUTPUTFORMAT,OUTPUTQUALIFIER)                      \
static JSBool CLASSNAME ## _ ## VARNAME ## _get (JSContext * cx, JSObject * obj, jsval id, jsval * vp)          \
{                                                                                                               \
    JSReflection::Context * c = JSReflection::Context::Find(cx);                                                \
    if (!c) { return JS_FALSE; }                                                                                \
    CLASSNAME * self;                                                                                           \
    RESULT_TYPE result;                                                                                         \
    if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; }               \
    result = self->VARNAME;                                                                                     \
    if (!c->ValueToJS((void*)&result,vp,OUTPUTFORMAT,(JSReflection::ClassInfo*)OUTPUTQUALIFIER)) { return JS_FALSE; }\
    return JS_TRUE;                                                                                             \
}

#define JS_INTERNAL_VAR_SETTER(CLASSNAME,VARNAME,VALUE_TYPE,VALUEFORMAT,VALUEQUALIFIER)                         \
static JSBool CLASSNAME ## _ ## VARNAME ## _set (JSContext * cx, JSObject * obj, jsval id, jsval * vp)          \
{                                                                                                               \
    JSReflection::Context * c = JSReflection::Context::Find(cx);                                                \
    if (!c) { return JS_FALSE; }                                                                                \
    CLASSNAME * self;                                                                                           \
    VALUE_TYPE value;                                                                                           \
    if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; }               \
    if (!c->ValueFromJS(vp,&value,VALUEFORMAT,VALUEQUALIFIER)) { return JS_FALSE; }                             \
    self->VARNAME = value;                                                                                      \
    return JS_TRUE;                                                                                             \
}

#define JS_RW_VARIABLE(CLASSNAME,VARNAME,VALUE_TYPE,VALUEFORMAT,VALUEQUALIFIER)                                 \
    JS_INTERNAL_VAR_GETTER(CLASSNAME,VARNAME,VALUE_TYPE,VALUEFORMAT,VALUEQUALIFIER)                             \
    JS_INTERNAL_VAR_SETTER(CLASSNAME,VARNAME,VALUE_TYPE,VALUEFORMAT,VALUEQUALIFIER)                             \
    static JSReflection::PropertyInfo CLASSNAME ## _ ## VARNAME ## _var                                         \
                        ( &CLASSNAME ## _ClassInfo::Properties, #VARNAME,                                       \
                            &CLASSNAME ## _ ## VARNAME ## _get,                                                 \
                            &CLASSNAME ## _ ## VARNAME ## _set);                                                \

#define JS_RO_VARIABLE(CLASSNAME,VARNAME,VALUE_TYPE,VALUEFORMAT,VALUEQUALIFIER)                                 \
    JS_INTERNAL_VAR_GETTER(CLASSNAME,VARNAME,VALUE_TYPE,VALUEFORMAT,VALUEQUALIFIER)                             \
    static JSReflection::PropertyInfo CLASSNAME ## _ ## VARNAME ## _var                                         \
                        ( &CLASSNAME ## _ClassInfo::Properties, #VARNAME,                                       \
                            &CLASSNAME ## _ ## VARNAME ## _get,0);

#define JS_GLOBAL_FUNCTION_0(FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER)                   \
  static JSBool FUNCNAME ## _jscallback (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )\
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      RESULT_TYPE result;                                                                           \
                                                                                                    \
      if (argc != 0) { return JS_FALSE; }                                                           \
                                                                                                    \
      result = FUNCNAME(c);                                                                         \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo FUNCNAME ## _jsfunction                                         \
                          (&JSReflection::Context::PublicFunctions, #FUNCNAME, &FUNCNAME ## _jscallback, 0);

#define JS_GLOBAL_FUNCTION_1(FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,                   \
                                          A1_TYPE,A1_FORMAT,A1_QUALIFIER)                           \
  static JSBool FUNCNAME ## _jscallback (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )\
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
                                                                                                    \
      if (argc != 1) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = FUNCNAME(c,a1);                                                                      \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo FUNCNAME ## _jsfunction                                         \
                          (&JSReflection::Context::PublicFunctions, #FUNCNAME, &FUNCNAME ## _jscallback, 1);

#define JS_GLOBAL_FUNCTION_2(FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,                   \
                                          A1_TYPE,A1_FORMAT,A1_QUALIFIER,                           \
                                          A2_TYPE,A2_FORMAT,A2_QUALIFIER)                           \
  static JSBool FUNCNAME ## _jscallback (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )\
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
                                                                                                    \
      if (argc != 2) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = FUNCNAME(c,a1,a2);                                                                   \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo FUNCNAME ## _jsfunction                                         \
                          (&JSReflection::Context::PublicFunctions, #FUNCNAME, &FUNCNAME ## _jscallback, 2);

#define JS_GLOBAL_FUNCTION_3(FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,                   \
                                          A1_TYPE,A1_FORMAT,A1_QUALIFIER,                           \
                                          A2_TYPE,A2_FORMAT,A2_QUALIFIER,                           \
                                          A3_TYPE,A3_FORMAT,A3_QUALIFIER)                           \
  static JSBool FUNCNAME ## _jscallback (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )\
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
                                                                                                    \
      if (argc != 3) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = FUNCNAME(c,a1,a2,a3);                                                                \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo FUNCNAME ## _jsfunction                                         \
                          (&JSReflection::Context::PublicFunctions, #FUNCNAME, &FUNCNAME ## _jscallback, 3);

#define JS_GLOBAL_FUNCTION_4(FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,                   \
                                          A1_TYPE,A1_FORMAT,A1_QUALIFIER,                           \
                                          A2_TYPE,A2_FORMAT,A2_QUALIFIER,                           \
                                          A3_TYPE,A3_FORMAT,A3_QUALIFIER,                           \
                                          A4_TYPE,A4_FORMAT,A4_QUALIFIER)                           \
  static JSBool FUNCNAME ## _jscallback (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )\
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
      A4_TYPE a4;                                                                                   \
                                                                                                    \
      if (argc != 4) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+3,&a4,A4_FORMAT,A4_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = FUNCNAME(c,a1,a2,a3,a4);                                                             \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo FUNCNAME ## _jsfunction                                         \
                          (&JSReflection::Context::PublicFunctions, #FUNCNAME, &FUNCNAME ## _jscallback, 4);

#define JS_GLOBAL_FUNCTION_5(FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,                   \
                                          A1_TYPE,A1_FORMAT,A1_QUALIFIER,                           \
                                          A2_TYPE,A2_FORMAT,A2_QUALIFIER,                           \
                                          A3_TYPE,A3_FORMAT,A3_QUALIFIER,                           \
                                          A4_TYPE,A4_FORMAT,A4_QUALIFIER,                           \
                                          A5_TYPE,A5_FORMAT,A5_QUALIFIER)                           \
  static JSBool FUNCNAME ## _jscallback (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )\
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
      A4_TYPE a4;                                                                                   \
      A5_TYPE a5;                                                                                   \
                                                                                                    \
      if (argc != 5) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+3,&a4,A4_FORMAT,A4_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+4,&a5,A5_FORMAT,A5_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = FUNCNAME(c,a1,a2,a3,a4,a5);                                                          \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo FUNCNAME ## _jsfunction                                         \
                          (&JSReflection::Context::PublicFunctions, #FUNCNAME, &FUNCNAME ## _jscallback, 5);



#define JS_NOCONTEXT_GLOBAL_FUNCTION_0(FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER)         \
  static JSBool FUNCNAME ## _jscallback (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )\
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      RESULT_TYPE result;                                                                           \
                                                                                                    \
      if (argc != 0) { return JS_FALSE; }                                                           \
                                                                                                    \
      result = FUNCNAME();                                                                          \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo FUNCNAME ## _jsfunction                                         \
                          (&JSReflection::Context::PublicFunctions, #FUNCNAME, &FUNCNAME ## _jscallback, 0);

#define JS_NOCONTEXT_GLOBAL_FUNCTION_1(FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,         \
                                          A1_TYPE,A1_FORMAT,A1_QUALIFIER)                           \
  static JSBool FUNCNAME ## _jscallback (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )\
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
                                                                                                    \
      if (argc != 1) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = FUNCNAME(a1);                                                                        \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo FUNCNAME ## _jsfunction                                         \
                          (&JSReflection::Context::PublicFunctions, #FUNCNAME, &FUNCNAME ## _jscallback, 1);

#define JS_NOCONTEXT_GLOBAL_FUNCTION_2(FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,         \
                                          A1_TYPE,A1_FORMAT,A1_QUALIFIER,                           \
                                          A2_TYPE,A2_FORMAT,A2_QUALIFIER)                           \
  static JSBool FUNCNAME ## _jscallback (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )\
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
                                                                                                    \
      if (argc != 2) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = FUNCNAME(a1,a2);                                                                     \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo FUNCNAME ## _jsfunction                                         \
                          (&JSReflection::Context::PublicFunctions, #FUNCNAME, &FUNCNAME ## _jscallback, 2);

#define JS_NOCONTEXT_GLOBAL_FUNCTION_3(FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,         \
                                          A1_TYPE,A1_FORMAT,A1_QUALIFIER,                           \
                                          A2_TYPE,A2_FORMAT,A2_QUALIFIER,                           \
                                          A3_TYPE,A3_FORMAT,A3_QUALIFIER)                           \
  static JSBool FUNCNAME ## _jscallback (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )\
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
                                                                                                    \
      if (argc != 3) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = FUNCNAME(a1,a2,a3);                                                                  \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo FUNCNAME ## _jsfunction                                         \
                          (&JSReflection::Context::PublicFunctions, #FUNCNAME, &FUNCNAME ## _jscallback, 3);

#define JS_NOCONTEXT_GLOBAL_FUNCTION_4(FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,         \
                                          A1_TYPE,A1_FORMAT,A1_QUALIFIER,                           \
                                          A2_TYPE,A2_FORMAT,A2_QUALIFIER,                           \
                                          A3_TYPE,A3_FORMAT,A3_QUALIFIER,                           \
                                          A4_TYPE,A4_FORMAT,A4_QUALIFIER)                           \
  static JSBool FUNCNAME ## _jscallback (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )\
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
      A4_TYPE a4;                                                                                   \
                                                                                                    \
      if (argc != 4) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+3,&a4,A4_FORMAT,A4_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = FUNCNAME(a1,a2,a3,a4);                                                               \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo FUNCNAME ## _jsfunction                                         \
                          (&JSReflection::Context::PublicFunctions, #FUNCNAME, &FUNCNAME ## _jscallback, 4);

#define JS_NOCONTEXT_GLOBAL_FUNCTION_5(FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,         \
                                          A1_TYPE,A1_FORMAT,A1_QUALIFIER,                           \
                                          A2_TYPE,A2_FORMAT,A2_QUALIFIER,                           \
                                          A3_TYPE,A3_FORMAT,A3_QUALIFIER,                           \
                                          A4_TYPE,A4_FORMAT,A4_QUALIFIER,                           \
                                          A5_TYPE,A5_FORMAT,A5_QUALIFIER)                           \
  static JSBool FUNCNAME ## _jscallback (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )\
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
      A4_TYPE a4;                                                                                   \
      A5_TYPE a5;                                                                                   \
                                                                                                    \
      if (argc != 5) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+3,&a4,A4_FORMAT,A4_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+4,&a5,A5_FORMAT,A5_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = FUNCNAME(a1,a2,a3,a4,a5);                                                            \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo FUNCNAME ## _jsfunction                                         \
                          (&JSReflection::Context::PublicFunctions, #FUNCNAME, &FUNCNAME ## _jscallback, 5);



#define JS_OBJECT_FUNCTION_0(CLASSNAME,FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER)         \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      RESULT_TYPE result;                                                                           \
                                                                                                    \
      if (argc != 0) { return JS_FALSE; }                                                           \
                                                                                                    \
      result = self->FUNCNAME(c);                                                                   \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,0);

#define JS_OBJECT_FUNCTION_1(CLASSNAME,FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,         \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
                                                                                                    \
      if (argc != 1) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = self->FUNCNAME(c,a1);                                                                \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,1);

#define JS_OBJECT_FUNCTION_2(CLASSNAME,FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,         \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
                                                                                                    \
      if (argc != 2) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = self->FUNCNAME(c,a1,a2);                                                             \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,2);

#define JS_OBJECT_FUNCTION_3(CLASSNAME,FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,         \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER,                                  \
                                   A3_TYPE,A3_FORMAT,A3_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
                                                                                                    \
      if (argc != 3) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = self->FUNCNAME(c,a1,a2,a3);                                                          \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,3);

#define JS_OBJECT_FUNCTION_4(CLASSNAME,FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,         \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER,                                  \
                                   A3_TYPE,A3_FORMAT,A3_QUALIFIER,                                  \
                                   A4_TYPE,A4_FORMAT,A4_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
      A4_TYPE a4;                                                                                   \
                                                                                                    \
      if (argc != 4) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+3,&a4,A4_FORMAT,A4_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = self->FUNCNAME(c,a1,a2,a3,a4);                                                       \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,4);

#define JS_OBJECT_FUNCTION_5(CLASSNAME,FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,         \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER,                                  \
                                   A3_TYPE,A3_FORMAT,A3_QUALIFIER,                                  \
                                   A4_TYPE,A4_FORMAT,A4_QUALIFIER,                                  \
                                   A5_TYPE,A5_FORMAT,A5_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
      A4_TYPE a4;                                                                                   \
      A5_TYPE a5;                                                                                   \
                                                                                                    \
      if (argc != 5) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+3,&a4,A4_FORMAT,A4_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+4,&a5,A5_FORMAT,A5_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = self->FUNCNAME(c,a1,a2,a3,a4,a5);                                                    \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,5);



#define JS_OBJECT_VOID_FUNCTION_0(CLASSNAME,FUNCNAME)                                               \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
                                                                                                    \
      if (argc != 0) { return JS_FALSE; }                                                           \
                                                                                                    \
      self->FUNCNAME(c);                                                                            \
      *r = JSVAL_VOID;                                                                              \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,0);

#define JS_OBJECT_VOID_FUNCTION_1(CLASSNAME,FUNCNAME,                                               \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      A1_TYPE a1;                                                                                   \
                                                                                                    \
      if (argc != 1) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      self->FUNCNAME(c,a1);                                                                         \
      *r = JSVAL_VOID;                                                                              \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,1);

#define JS_OBJECT_VOID_FUNCTION_2(CLASSNAME,FUNCNAME,                                               \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
                                                                                                    \
      if (argc != 2) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      self->FUNCNAME(c,a1,a2);                                                                      \
      *r = JSVAL_VOID;                                                                              \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,2);

#define JS_OBJECT_VOID_FUNCTION_3(CLASSNAME,FUNCNAME,                                               \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER,                                  \
                                   A3_TYPE,A3_FORMAT,A3_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
                                                                                                    \
      if (argc != 3) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      self->FUNCNAME(c,a1,a2,a3);                                                                   \
      *r = JSVAL_VOID;                                                                              \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,3);

#define JS_OBJECT_VOID_FUNCTION_4(CLASSNAME,FUNCNAME,                                               \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER,                                  \
                                   A3_TYPE,A3_FORMAT,A3_QUALIFIER,                                  \
                                   A4_TYPE,A4_FORMAT,A4_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
      A4_TYPE a4;                                                                                   \
                                                                                                    \
      if (argc != 4) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+3,&a4,A4_FORMAT,A4_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      self->FUNCNAME(c,a1,a2,a3,a4);                                                                \
      *r = JSVAL_VOID;                                                                              \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,4);

#define JS_OBJECT_VOID_FUNCTION_5(CLASSNAME,FUNCNAME,                                               \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER,                                  \
                                   A3_TYPE,A3_FORMAT,A3_QUALIFIER,                                  \
                                   A4_TYPE,A4_FORMAT,A4_QUALIFIER,                                  \
                                   A5_TYPE,A5_FORMAT,A5_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
      A4_TYPE a4;                                                                                   \
      A5_TYPE a5;                                                                                   \
                                                                                                    \
      if (argc != 5) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+3,&a4,A4_FORMAT,A4_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+4,&a5,A5_FORMAT,A5_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      self->FUNCNAME(c,a1,a2,a3,a4,a5);                                                             \
      *r = JSVAL_VOID;                                                                              \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,5);



#define JS_NOCONTEXT_OBJECT_FUNCTION_0(CLASSNAME,FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER)\
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      RESULT_TYPE result;                                                                           \
                                                                                                    \
      if (argc != 0) { return JS_FALSE; }                                                           \
                                                                                                    \
      result = self->FUNCNAME();                                                                    \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,0);

#define JS_NOCONTEXT_OBJECT_FUNCTION_1(CLASSNAME,FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,\
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
                                                                                                    \
      if (argc != 1) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = self->FUNCNAME(a1);                                                                  \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,1);

#define JS_NOCONTEXT_OBJECT_FUNCTION_2(CLASSNAME,FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,\
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
                                                                                                    \
      if (argc != 2) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = self->FUNCNAME(a1,a2);                                                               \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,2);

#define JS_NOCONTEXT_OBJECT_FUNCTION_3(CLASSNAME,FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,\
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER,                                  \
                                   A3_TYPE,A3_FORMAT,A3_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
                                                                                                    \
      if (argc != 3) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = self->FUNCNAME(a1,a2,a3);                                                            \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,3);

#define JS_NOCONTEXT_OBJECT_FUNCTION_4(CLASSNAME,FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,\
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER,                                  \
                                   A3_TYPE,A3_FORMAT,A3_QUALIFIER,                                  \
                                   A4_TYPE,A4_FORMAT,A4_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
      A4_TYPE a4;                                                                                   \
                                                                                                    \
      if (argc != 4) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+3,&a4,A4_FORMAT,A4_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = self->FUNCNAME(a1,a2,a3,a4);                                                         \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,4);

#define JS_NOCONTEXT_OBJECT_FUNCTION_5(CLASSNAME,FUNCNAME,RESULT_TYPE,RESULT_FORMAT,RESULT_QUALIFIER,\
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER,                                  \
                                   A3_TYPE,A3_FORMAT,A3_QUALIFIER,                                  \
                                   A4_TYPE,A4_FORMAT,A4_QUALIFIER,                                  \
                                   A5_TYPE,A5_FORMAT,A5_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      RESULT_TYPE result;                                                                           \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
      A4_TYPE a4;                                                                                   \
      A5_TYPE a5;                                                                                   \
                                                                                                    \
      if (argc != 5) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+3,&a4,A4_FORMAT,A4_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+4,&a5,A5_FORMAT,A5_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      result = self->FUNCNAME(a1,a2,a3,a4,a5);                                                      \
      if (!c->ValueToJS(&result,r,RESULT_FORMAT,RESULT_QUALIFIER)) { return JS_FALSE; }             \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,5);





#define JS_NOCONTEXT_OBJECT_VOID_FUNCTION_0(CLASSNAME,FUNCNAME)                                     \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
                                                                                                    \
      if (argc != 0) { return JS_FALSE; }                                                           \
                                                                                                    \
      self->FUNCNAME();                                                                             \
      *r = JSVAL_VOID;                                                                              \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,0);

#define JS_NOCONTEXT_OBJECT_VOID_FUNCTION_1(CLASSNAME,FUNCNAME,                                     \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      A1_TYPE a1;                                                                                   \
                                                                                                    \
      if (argc != 1) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      self->FUNCNAME(a1);                                                                           \
      *r = JSVAL_VOID;                                                                              \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,1);

#define JS_NOCONTEXT_OBJECT_VOID_FUNCTION_2(CLASSNAME,FUNCNAME,                                     \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
                                                                                                    \
      if (argc != 2) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      self->FUNCNAME(a1,a2);                                                                        \
      *r = JSVAL_VOID;                                                                              \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,2);

#define JS_NOCONTEXT_OBJECT_VOID_FUNCTION_3(CLASSNAME,FUNCNAME,                                     \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER,                                  \
                                   A3_TYPE,A3_FORMAT,A3_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
                                                                                                    \
      if (argc != 3) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      self->FUNCNAME(a1,a2,a3);                                                                     \
      *r = JSVAL_VOID;                                                                              \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,3);

#define JS_NOCONTEXT_OBJECT_VOID_FUNCTION_4(CLASSNAME,FUNCNAME,                                     \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER,                                  \
                                   A3_TYPE,A3_FORMAT,A3_QUALIFIER,                                  \
                                   A4_TYPE,A4_FORMAT,A4_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
      A4_TYPE a4;                                                                                   \
                                                                                                    \
      if (argc != 4) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+3,&a4,A4_FORMAT,A4_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      self->FUNCNAME(a1,a2,a3,a4);                                                                  \
      *r = JSVAL_VOID;                                                                              \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,4);

#define JS_NOCONTEXT_OBJECT_VOID_FUNCTION_5(CLASSNAME,FUNCNAME,                                     \
                                   A1_TYPE,A1_FORMAT,A1_QUALIFIER,                                  \
                                   A2_TYPE,A2_FORMAT,A2_QUALIFIER,                                  \
                                   A3_TYPE,A3_FORMAT,A3_QUALIFIER,                                  \
                                   A4_TYPE,A4_FORMAT,A4_QUALIFIER,                                  \
                                   A5_TYPE,A5_FORMAT,A5_QUALIFIER)                                  \
  static JSBool CLASSNAME ## _ ## FUNCNAME ## _jscallback                                           \
                  (JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * r )            \
  {                                                                                                 \
      JSReflection::Context * c = JSReflection::Context::Find(cx);                                  \
      if (!c) { return JS_FALSE; }                                                                  \
      CLASSNAME * self;                                                                             \
      if (!c->ValueFromJSObject(obj,(void**)&self,"%!x",CLASSNAME::ClassInfo)) { return JS_FALSE; } \
                                                                                                    \
      A1_TYPE a1;                                                                                   \
      A2_TYPE a2;                                                                                   \
      A3_TYPE a3;                                                                                   \
      A4_TYPE a4;                                                                                   \
      A5_TYPE a5;                                                                                   \
                                                                                                    \
      if (argc != 5) { return JS_FALSE; }                                                           \
      if (!c->ValueFromJS(argv+0,&a1,A1_FORMAT,A1_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+1,&a2,A2_FORMAT,A2_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+2,&a3,A3_FORMAT,A3_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+3,&a4,A4_FORMAT,A4_QUALIFIER)) { return JS_FALSE; }                  \
      if (!c->ValueFromJS(argv+4,&a5,A5_FORMAT,A5_QUALIFIER)) { return JS_FALSE; }                  \
                                                                                                    \
      self->FUNCNAME(a1,a2,a3,a4,a5);                                                               \
      *r = JSVAL_VOID;                                                                              \
      return JS_TRUE;                                                                               \
  }                                                                                                 \
  static JSReflection::FunctionInfo CALLNAME ## _ ## FUNCNAME ## _jsfunction                        \
                  (&CLASSNAME ## _ClassInfo::Functions, #FUNCNAME, &CLASSNAME ## _ ## FUNCNAME ## _jscallback,5);





