/***************************************************************************
*   Copyright (C) 2004 by ludovic p                                       *
*   pludov@nnx.com                                                        *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef JSREFLECT_H
#define JSREFLECT_H 1

/*
Skeleton for any exported class :
 
class Dummy : public Object {
    JS_EXPORTED_CLASS
 
public:
 
    Dummy(Context * c, ... );
};
 
The js_dummy.cpp will contain
 
JS_EXPORT_CLASS(Dummy,FatherOfDummy)
 
 
*/

// just for jsapi.h to work
#define XP_UNIX

#include "jsapi.h"
#include "jsmacros.h"
#include <string>
#define JS_EXPORTED_CLASS                                                               \
    public:                                                                             \
        static JSReflection::ClassInfo * ClassInfo;                                     \
        virtual const JSReflection::ClassInfo * GetClassId() const;

    /// removed : template JSReflection::SpecificClassInfo<name>;
#define JS_EXPORT_CLASS(name,parentclass)                                               \
    class name ## _ClassInfo:public JSReflection::SpecificClassInfo<name> {    \
    public:                                                                             \
        name ## _ClassInfo(): JSReflection::SpecificClassInfo<name>( #name , &parentclass::ClassInfo) {} \
    };                                                                                  \
    JSReflection::ClassInfo * name::ClassInfo = new name ## _ClassInfo ();              \
    const JSReflection::ClassInfo * name::GetClassId() const { return name::ClassInfo; };

#define jsrlog(format,...) jsr_output_log(__FILE__ , __LINE__ ,format,##__VA_ARGS__)

bool jsr_debug_enabled();
void jsr_output_log(const char * file,int line, const char *format, ...);

namespace JSReflection
{
    class Context;
    class Object;
    class ClassInfo;
    class Event;
    class Closure;


    /** Export a property to JS.
     *
     * A property must be linked to a ClassInfo. It can be readonly or readwrite.
     *
     * - Use JS_RO_PROPERTY & JS_RW_PROPERTY macros to define property (i.e. not instance variable)
     * - Use JS_RO_VARIABLE & JS_RW_VARIABLE macros to define instance variable
     *
     * PropertyInfo objects need to be instanciated statically 
     * (i.e. before first Context is instanciated.)
     */
    class PropertyInfo
    {
            friend class ClassInfo;
            friend class Context;
        protected:
            const char * Name;
            PropertyInfo * Next;
            JSPropertyOp Get, Set;
        public:
            PropertyInfo( PropertyInfo ** list, const char * name, JSPropertyOp get, JSPropertyOp set = 0 );
    };

    /** Export a C/C++ function to JS.
     *
     * A function must be linked to a ClassInfo or to the global Context::PublicFunctions.
     *
     * See the various macros available in jsmacros.h for simple function declaration.
     *
     * FunctionInfo objects need to be instanciated statically 
     * (i.e. before first Context is instanciated.)
     */
    class FunctionInfo
    {
            friend class ClassInfo;
            friend class Context;
        protected:
            const char * Name;
            JSNative Func;
            FunctionInfo * Next;
            int NArgs;
        public:
            FunctionInfo( FunctionInfo ** list, const char * name, JSNative func, int nargs );
	    void Debug() const;
    };

    class ClassInfo
    {
	    friend class Context;
        protected:
            static int Count;
            static ClassInfo * First;
        protected:
            const char * Name;
            int Id;

            ClassInfo * Next;
            ClassInfo * * Ancestor;

            virtual const PropertyInfo * GetProperties() const = 0;
            virtual const FunctionInfo * GetFunctions() const = 0;

            void FieldDebug() const;
        public:
            ClassInfo( const char * name,
                       ClassInfo * * ancestor );

            virtual ~ClassInfo();

            ClassInfo * GetAncestor() const;

            void Debug() const;

            const char * GetName() const;

            int GetId() const;

            bool SubClassOf( const ClassInfo * m ) const;
            virtual void* CastToSubClass( Object * m ) = 0;
            virtual Object* CastFromSubClass( void * m ) = 0;

            JSObject * CreateModel( Context * c ) const;

            static ClassInfo * FindByName( const char * name );

            static int GetClassCount();
    };

    template <class M>
    class SpecificClassInfo: public ClassInfo
    {
        public:
            SpecificClassInfo( const char * name, ClassInfo ** ancestor ) : ClassInfo( name, ancestor )
            { }

            virtual ~SpecificClassInfo()
            { }


            static PropertyInfo * Properties;
            static FunctionInfo * Functions;

            virtual const PropertyInfo * GetProperties() const
            {
                return Properties;
            }

            virtual const FunctionInfo * GetFunctions() const
            {
                return Functions;
            }

            virtual void* CastToSubClass( Object * m )
            {
                if ( !m )
                {
                    return 0;
                }
                else
                {
                    return ( void* ) ( ( M* ) m );
                }
            }

            virtual Object* CastFromSubClass( void * m )
            {
                if ( !m )
                {
                    return 0;
                }
                else
                {
                    M * u;
                    u = ( M* ) m;
                    return u;
                }
            }
    };

    template <class M>
    PropertyInfo * SpecificClassInfo<M>::Properties = 0;

    template <class M>
    FunctionInfo * SpecificClassInfo<M>::Functions = 0;

    class JSSourceInfo
    {
	std::string name;
	int line;
    public:
    	JSSourceInfo();
	JSSourceInfo(const std::string & name,int line);
	~JSSourceInfo();
	
	std::string getName() const;
	void setName(const std::string & name);
	
	int getLine() const;
	void setLine(int i);
	
	static JSSourceInfo unknown;
    };
    

    class Context
    {
            friend class Object;
            friend class ClassInfo;
            Object * First, * Last;
	    Object * global;
	    
            JSContext * JSGlobalContext;
            JSObject * JSGlobalObject;
            JSObject ** Models;
	    JSRuntime * runtime;
		    
            /** Create or find the model for a class info */
            JSObject * GetModel( const ClassInfo * c );
            /** Create a new unassociated jsobject for a classinfo */
            JSObject * Context::NewInstance( const ClassInfo * cd );
        public:

            Context( JSRuntime * js_runtime, int stack_size );
            ~Context();

            bool ExecuteScriptFile( const char * filename, Object * root = 0 );
	    bool ExecuteScript( const char * script, Object * root = 0, jsval * ret = 0,const JSSourceInfo & jsfi = JSSourceInfo::unknown);
	    bool ExecuteScript( const char * script, JSObject * root, jsval * ret = 0,const JSSourceInfo & jsfi = JSSourceInfo::unknown);
	    
	    void newZombi(JSObject * o,Object * initial);
	    
            JSObject * CreateEmptyModel();

            bool ValueFromJSObject( JSObject * in, void * out, const char * format, ClassInfo * specifier = 0 );
            bool ValueFromJS( jsval * in, void * out, const char * format, ClassInfo * specifier = 0 );
            bool ValueToJS( void * in, jsval * out, const char * format, ClassInfo * specifier = 0 );

            static Context * Find( JSContext * c );
            JSContext * GetJSContext() const;
                    
            Object * GetAssociatedObject( JSObject*o ) const;
	    Object * GetGlobal() const;
            static FunctionInfo * PublicFunctions;
	    
	    void DumpPredefined();
	    void dumpRoots();
    };

    class Object
    {
	
	    JS_EXPORTED_CLASS
	public:
	    int debug;
        protected:
	    friend class Context;
	    friend class Event;
	    friend class EventFIFO;
	    bool debugFlag;
	    
		    
            // Context
            Context * ReflectionContext;

            // Linked list for the context
            Object * Next, * Prev;

            // JSObject counterpart
            JSObject * Counterpart;

            // Counterpart is jsroot ?
            bool PreserveCounterpart;

	    Event * FirstTargeting, * LastTargeting;
	    Event * FirstSourcing, * LastSourcing;
	    
	    // ZT list of object listening for.
	    Object ** RelayEventTo;
	    const char ** RelayEventId;
	    
	    void AddRelay(Object * ,const char * id);
	    void DelRelay(Object * );
	    
	    // ZT list of object listening to
	    Object ** Relayed;
	    void AddRelayed(Object *);
	    void DelRelayed(Object *);
	    
	    // Called each time a relay is lost/unlistened
	    virtual void RelayedLost(Object * o);
	    
	    // Called each time an object relaying its events to this is lost
	    virtual void RelayerLost(Object * o);
	    
	    virtual void HandleEvent(Event * a);
	    
	public:    
	    bool debugGC() const;

	    Object( Context * c, bool preserved = false, int objectid = -1 );
            virtual ~Object();

            JSObject * GetJSObject( bool autocreate = true );

            // Only for builtin management.
            void SetCounterpart( JSObject * o );
            void DettachContext();

	    virtual bool isDebugZombi() const;
	    
	    bool evaluate(const char * value,const JSSourceInfo & fsfi = JSSourceInfo::unknown);
	    bool evaluate(const char * value,void * retvalue, const char * retformat,
	    		JSReflection::ClassInfo * retspecifier = 0,const JSSourceInfo & fsfi = JSSourceInfo::unknown);
	    bool setProperty(const char * name,const char * jsvalue,const JSSourceInfo & fsfi = JSSourceInfo::unknown);
	    bool setProperty(const char * name,Object * v);
	    bool setProperty(const char * name,jsval v);
	    
            // Direct control over javascript garbage collection
            void use(int count);
       
	    // "call" object will receive "id" event targeting this event
	    void Listen(const char * id, Object * call);
	    void Listen(const char * id,Object * t,Closure * o);
	    void Unlisten(Object * call);
	    
            const char * GetClassName() const;
	    
	    bool getDebugGC() const;
	    void setDebugGC(bool b);
            
            Context * GetContext() const;
	    
	    /// Args are value, MetaClass, value, MetaClass
	    bool call(const char * name, void * retvalue, const char * retformat,
	    		JSReflection::ClassInfo * retspecifier, const char * formats, ...);
    };

    class JS_String
    {
            bool NeedFree;
            const char * Value;
        public:
            JS_String();
            ~JS_String();
            const JS_String & operator=( const char * value );
            void Copy( const char * value, int length );
            const JS_String & operator=(const std::string & cpy);
	    operator const char *() const;
	    operator std::string() const;
	    
            const char * data() const;
            void SetFree( bool value );
    };

    class Closure;
    
    class EventFIFO: public Object
    {
            friend class Event;
            
            JS_EXPORTED_CLASS

            Event * First, * Last;
        public:
            EventFIFO(Context * context);
            virtual ~EventFIFO();
	    
	    Event * GetFirstEvent() const;
	    Event * GetLastEvent() const;
	    
            // Return true if events are still presents
            bool ProceedEvent();
    };

    class Event: public Object
    {
	    friend class Object;
	    friend class EventFIFO;
            JS_EXPORTED_CLASS

	    // If EventId starts with '#', it is allocated/deallocated on the heap
	    // The # is ignored for comparison.
	    const char * EventId;
	    
            EventFIFO * Father;
            Event * NextInFIFO, * PrevInFIFO;

	    Object * Source;
	    Event * NextInSource, * PrevInSource;
	    
            Object * Target;
            Event * NextInTarget, * PrevInTarget;
	    
	protected:
	    // Create new unattached Event
            virtual Event * clone() const = 0;
	    
	    virtual void SourceLost();
	    
	    virtual void TargetLost();
	    
	    // Calculate target just before treating.
	    virtual void Dispatch();
	    
	    // Called just after the event has been handled.
	    virtual void Treated();
	    
	    void SetRoute(Object * source, Object * target, Event * rightBefore);
	    
        public:
	    Event( EventFIFO * f, const char * uniqId );
            virtual ~Event();
	    
	    Object * GetSource() const;
	    Object * GetTarget() const;
	    
	    Event * GetNextInFifo() const;
	    
	    virtual void Debug() const;
	    bool IsA(const char * id) const;
    };

    class EventSource
    {
            friend class Event;
            // Events which will be triggered on this source.
            Event * First, * Last;
            
            Object* Parent;

            Closure * closure;
        public:
            EventSource( Object * parent );
            ~EventSource();
            void Trigger( Event * e ) const;

            operator Closure*() const;
            EventSource& operator=( Closure* );
    };

    class Closure: public Object
    {
            JS_EXPORTED_CLASS
	    
	    virtual void HandleEvent(Event * e);
	    
	    // Called each time a relay is lost/unlistened
	    virtual void RelayedLost();
	    
	    // Called each time an object relaying its events to this is lost
	    virtual void RelayerLost();
	     
        public:
            Closure( Context * c );
            virtual ~Closure();
	    
            virtual void Call( Object * o, Object * e ) = 0;
    };

    class JSClosure: public Closure
    {
	JS_EXPORTED_CLASS
	JSFunction * Function;
	jsval * Args;
	int ArgCount;
    public:
	JSClosure( Context * c, JSFunction * call, uintN argc, jsval * argv );
	virtual ~JSClosure();
	virtual void Call( Object * o, jsval arg);
	virtual void Call( Object * o, Object * e );
    };
    
    std::string decodeBinaryPath(const std::string & utf8);
    
    // Convert a binary path into an utf8 string.
    std::string displayBinaryPath(const std::string & utf8);
};

#endif
